import { dirname, relative } from "pathe";
import { ImportState } from "../classes/ImportState.js";
import { resolveExportName } from "../util/functions/resolveExportName.js";
export class FileContent {
    constructor(config, file) {
        this.config = config;
        this._imports = new Map();
        this._importStatements = [];
        this._declarationNames = new Set();
        this._declarations = [];
        this._reExports = { type: [], value: [] };
        this.file = file;
    }
    _getImportState(file) {
        if (file === this.file) {
            return {
                getImport: n => n.exportName,
            };
        }
        const cachedImportState = this._imports.get(file);
        if (cachedImportState !== undefined)
            return cachedImportState;
        const newImportState = new ImportState(file);
        this._imports.set(file, newImportState);
        return newImportState;
    }
    _addImportStatement(importStatement) {
        if (this._importStatements.some(s => s.module === importStatement.module &&
            s.name === importStatement.name &&
            s.as === importStatement.as &&
            s.type === importStatement.type &&
            s.isType === importStatement.isType))
            return;
        this._importStatements.push(importStatement);
    }
    pushDeclaration(typeID, mode, declaration) {
        const identifierName = resolveExportName(this.config, typeID);
        if (!this._declarationNames.has(identifierName)) {
            this._declarationNames.add(identifierName);
            const declarationLines = declaration(identifierName, {
                getImport: (id) => this._getImportState(id.file).getImport(id),
                addImportStatement: (importStatement) => this._addImportStatement(importStatement),
                getDeclarationName: (id) => resolveExportName(this.config, id),
                getRelativePath: (id) => {
                    const relativePath = relative(dirname(this.file), id.file);
                    return `${relativePath[0] === "." ? "" : "./"}${relativePath.replace(/(\.d)?\.(t|j)sx?$/, "").replaceAll("\\", "/")}.js`;
                },
            });
            this._declarations.push(() => [...declarationLines, identifierName ? `export ${mode === "type" ? "type " : ""}{${identifierName}}` : ""]);
        }
        return {
            mode,
            file: this.file,
            hasPrimaryKey: this._declarations.some(line => line().some(l => l.startsWith("export const PrimaryKey"))),
            exportName: identifierName,
        };
    }
    pushReExport(destination, source) {
        const identifierName = resolveExportName(this.config, destination);
        if (this._declarationNames.has(identifierName))
            return;
        const importedName = this._getImportState(source.file).getImport(source);
        this._declarationNames.add(identifierName);
        this._reExports[source.mode].push({
            source: importedName,
            dest: identifierName,
        });
    }
    _getImportStatements() {
        //* Group import statements by module
        const groupedImportStatements = new Map();
        for (const importStatement of this._importStatements) {
            const group = groupedImportStatements.get(importStatement.module);
            if (group === undefined)
                groupedImportStatements.set(importStatement.module, [importStatement]);
            else
                group.push(importStatement);
        }
        const mergedImportStatements = new Map();
        //* For all import statements in a group, where the type is "named" and the names are the same (and they don't have an as), merge them into a single import statement
        for (const [module, group] of groupedImportStatements.entries()) {
            let newStatement;
            for (const statement of group) {
                if (statement.type !== "named" || statement.as !== undefined) {
                    const mergedGroup = mergedImportStatements.get(module);
                    if (mergedGroup === undefined) {
                        mergedImportStatements.set(module, [
                            {
                                ...statement,
                                names: [],
                            },
                        ]);
                    }
                    else {
                        mergedGroup.push({
                            ...statement,
                            names: [],
                        });
                    }
                }
                if (newStatement === undefined) {
                    newStatement = {
                        ...statement,
                        isType: statement.isType ?? false,
                        names: [[statement.name, statement.isType ?? false]],
                    };
                    continue;
                }
                const hasSameNameIndex = newStatement.names.findIndex(([name]) => name === statement.name);
                if (hasSameNameIndex === -1)
                    newStatement.names.push([statement.name, statement.isType ?? false]);
                else {
                    const sameName = newStatement.names[hasSameNameIndex];
                    let isType = sameName[1];
                    if (isType && !statement.isType)
                        isType = false;
                    newStatement.names[hasSameNameIndex] = [sameName[0], isType];
                }
            }
            if (newStatement === undefined)
                continue;
            //* If all the names in the statement are types, mark the statement as a type statement
            newStatement.isType = newStatement.names.every(([, isType]) => isType);
            const mergedGroup = mergedImportStatements.get(module);
            if (mergedGroup === undefined)
                mergedImportStatements.set(module, [newStatement]);
            else
                mergedGroup.push(newStatement);
        }
        //* Generate import statements
        const importStrings = [];
        for (const [module, group] of mergedImportStatements.entries()) {
            for (const statement of group) {
                if (statement.type === "default")
                    importStrings.push(`import ${statement.as ?? statement.name} from "${module}";`);
                else if (statement.type === "named") {
                    const names = statement.names
                        .sort(([a], [b]) => a.localeCompare(b))
                        .map(([name, isType]) => `${!statement.isType && isType ? "type " : ""}${name}`)
                        .join(",\n  ") || statement.name;
                    importStrings.push(`import${statement.isType ? " type" : ""} ${statement.as === undefined ? `{\n  ${names}\n}` : `{\n  ${names}\n} as ${statement.as}`} from "${module}";`);
                }
            }
        }
        return importStrings.join("\n");
    }
    getContent() {
        return `${[
            this._getImportStatements(),
            ...(this._imports.size > 0
                ? [
                    [...this._imports.values()]
                        .sort((a, b) => (a.file < b.file ? -1 : 1))
                        .map(imp => {
                        const relativePath = relative(dirname(this.file), imp.file);
                        return imp.getImportStatement(`${relativePath[0] === "." ? "" : "./"}${relativePath.replace(/(\.d)?\.(t|j)sx?$/, "").replaceAll("\\", "/")}.js`);
                    })
                        .join("\n"),
                ]
                : []),
            ...this._declarations.map(v => v().join("\n")),
            ...(this._reExports.type.length > 0
                ? [`export type {\n${this._reExports.type.map(t => (t.source === t.dest ? `  ${t.source},` : `  ${t.source} as ${t.dest},`)).join("\n")}\n}`]
                : []),
            ...(this._reExports.value.length > 0
                ? [`export {\n${this._reExports.value.map(t => (t.source === t.dest ? `  ${t.source},` : `  ${t.source} as ${t.dest},`)).join("\n")}\n}`]
                : []),
        ].join("\n\n")}\n`;
    }
}
//# sourceMappingURL=FileContent.js.map